@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -15,23 +15,23 @@
 // You should have received a copy of the GNU General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-#include "hash_sha2_384.h"
+#include "hash_sha2_512.h"
 
 namespace mobius
 {
 namespace crypt
 {
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-// initialization constants (FIPS.180-4 - section 5.3.4)
+// initialization constants (FIPS.180-4 - section 5.3.5)
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-static constexpr std::uint64_t H0_VALUE = 0xcbbb9d5dc1059ed8;
-static constexpr std::uint64_t H1_VALUE = 0x629a292a367cd507;
-static constexpr std::uint64_t H2_VALUE = 0x9159015a3070dd17;
-static constexpr std::uint64_t H3_VALUE = 0x152fecd8f70e5939;
-static constexpr std::uint64_t H4_VALUE = 0x67332667ffc00b31;
-static constexpr std::uint64_t H5_VALUE = 0x8eb44a8768581511;
-static constexpr std::uint64_t H6_VALUE = 0xdb0c2e0d64f98fa7;
-static constexpr std::uint64_t H7_VALUE = 0x47b5481dbefa4fa4;
+static constexpr std::uint64_t H0_VALUE = 0x6a09e667f3bcc908;
+static constexpr std::uint64_t H1_VALUE = 0xbb67ae8584caa73b;
+static constexpr std::uint64_t H2_VALUE = 0x3c6ef372fe94f82b;
+static constexpr std::uint64_t H3_VALUE = 0xa54ff53a5f1d36f1;
+static constexpr std::uint64_t H4_VALUE = 0x510e527fade682d1;
+static constexpr std::uint64_t H5_VALUE = 0x9b05688c2b3e6c1f;
+static constexpr std::uint64_t H6_VALUE = 0x1f83d9abfb41bd6b;
+static constexpr std::uint64_t H7_VALUE = 0x5be0cd19137e2179;
 
 //! \brief block size = 128 bytes (1024 bits) - FIPS.180-4 - section 1
 static constexpr int BLOCK_SIZE = 128;
@@ -51,7 +51,7 @@ rrot (std::uint64_t v, std::uint64_t n) noexcept
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief default constructor
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-hash_sha2_384::hash_sha2_384 () noexcept
+hash_sha2_512::hash_sha2_512 () noexcept
   : hash_block (BLOCK_SIZE),
     a_ (H0_VALUE),
     b_ (H1_VALUE),
@@ -69,7 +69,7 @@ hash_sha2_384::hash_sha2_384 () noexcept
 //! \brief reset hash value
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void
-hash_sha2_384::_reset () noexcept
+hash_sha2_512::_reset () noexcept
 {
   a_ = H0_VALUE;
   b_ = H1_VALUE;
@@ -88,7 +88,7 @@ hash_sha2_384::_reset () noexcept
 //! \see FIPS.180-4 - section 6.2
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void
-hash_sha2_384::_update_block (const mobius::bytearray& data) noexcept
+hash_sha2_512::_update_block (const mobius::bytearray& data) noexcept
 {
   std::uint64_t a = a_;
   std::uint64_t b = b_;
@@ -1201,9 +1201,9 @@ hash_sha2_384::_update_block (const mobius::bytearray& data) noexcept
 // representing the message size in bits (FIPS.180-4 - section 5.1.2)
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 mobius::bytearray
-hash_sha2_384::_evaluate (const mobius::bytearray& data)
+hash_sha2_512::_evaluate (const mobius::bytearray& data)
 {
-  hash_sha2_384 tmp (*this);
+  hash_sha2_512 tmp (*this);
 
   // represent message size in bits as a bytearray
   std::uint64_t bits = ((size_ + data.size ()) << 3);
@@ -1304,6 +1304,22 @@ hash_sha2_384::_evaluate (const mobius::bytearray& data)
     std::uint8_t (f_ >> 16),
     std::uint8_t (f_ >> 8),
     std::uint8_t (f_),
+    std::uint8_t (g_ >> 56),
+    std::uint8_t (g_ >> 48),
+    std::uint8_t (g_ >> 40),
+    std::uint8_t (g_ >> 32),
+    std::uint8_t (g_ >> 24),
+    std::uint8_t (g_ >> 16),
+    std::uint8_t (g_ >> 8),
+    std::uint8_t (g_),
+    std::uint8_t (h_ >> 56),
+    std::uint8_t (h_ >> 48),
+    std::uint8_t (h_ >> 40),
+    std::uint8_t (h_ >> 32),
+    std::uint8_t (h_ >> 24),
+    std::uint8_t (h_ >> 16),
+    std::uint8_t (h_ >> 8),
+    std::uint8_t (h_),
   };
 
   *this = tmp;
