@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -15,7 +15,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-#include <mobius/datetime/conv_iso_string.h>
+#include "conv_iso_string.h"
 #include <mobius/exception.inc>
 #include <stdexcept>
 #include <sstream>
@@ -26,11 +26,43 @@ namespace mobius
 namespace datetime
 {
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief convert date object to ISO 8601 string
+//! \brief Convert ISO 8601 formatted string to datetime
+//! \param str string
+//! \return datetime
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+datetime
+new_datetime_from_iso_string (const std::string& str)
+{
+  int year, month, day;
+  int hours, minutes, seconds;
+  const char *format = nullptr;
+
+  if (str.length () > 10 && str[10] == 'T')
+    format = "%04d-%02d-%02dT%02d:%02d:%02d";
+  else
+    format = "%04d-%02d-%02d %02d:%02d:%02d";
+
+  if (sscanf (
+        str.c_str (),
+        format,
+        &year,
+        &month,
+        &day,
+        &hours,
+        &minutes,
+        &seconds) == EOF
+     )
+    throw std::runtime_error (MOBIUS_EXCEPTION_MSG (("Invalid datetime string: " + str).c_str ()));
+
+  return datetime (year, month, day, hours, minutes, seconds);
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Convert date object to ISO 8601 string
 //! \param d date object
 //! \return date formatted as string
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 date_to_iso_string (const date& d)
 {
   char buffer[64] = {0};
@@ -42,11 +74,11 @@ date_to_iso_string (const date& d)
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief convert time object to ISO 8601 string
+//! \brief Convert time object to ISO 8601 string
 //! \param t time object
 //! \return time formatted as 'HH:MM:SS'
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 time_to_iso_string (const time& t)
 {
   char buffer[64] = {0};
@@ -58,11 +90,11 @@ time_to_iso_string (const time& t)
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief convert datetime object to ISO 8601 format
+//! \brief Convert datetime object to ISO 8601 format
 //! \param dt datetime object
 //! \return datetime formatted as 'YYYY-MM-DD HH:MM:SS'
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 datetime_to_iso_string (const datetime& dt)
 {
   std::string str;
@@ -73,28 +105,5 @@ datetime_to_iso_string (const datetime& dt)
   return str;
 }
 
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief convert ISO 8601 formatted string to datetime
-//! \param str string
-//! \return datetime
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const datetime
-datetime_from_iso_string (const std::string& str)
-{
-  std::istringstream stream (str);
-
-  int year, month, day;
-  int hours, minutes, seconds;
-  char sep1 = 0, sep2 = 0, sep3 = 0, sep4 = 0, sep5 = 0;
-
-  // !\todo parse timezone
-  if (!(stream >> year >> sep1 >> month >> sep2 >> day >> sep3
-        >> hours >> sep4 >> minutes >> sep5 >> seconds) ||
-      (sep1 != '-' || sep2 != '-' || sep3 != 'T' || sep4 != ':' || sep5 != ':'))
-    throw std::runtime_error (MOBIUS_EXCEPTION_MSG (("invalid datetime string: " + str).c_str ()));
-
-  return datetime (year, month, day, hours, minutes, seconds);
-}
-
 } // namespace datetime
 } // namespace mobius
