@@ -4,17 +4,17 @@ import mobius
 import datetime
 import sys
 
-def show_entry (r, level=0):
+def show_entry (r, path='/', level=0):
   is_folder = isinstance (r, mobius.io.folder)
   indent = ' ' * (level * 2)
 
-  if not r.is_valid or not r.exists:
-    print '%s? %s' % (indent, r.url)
+  if not r.exists:
+    print '%s? %s' % (indent, path)
     return
 
   # show metadata
   print
-  print '%s%s %s' % (indent, '+' if is_folder else '.', r.url)
+  print '%s%s %s' % (indent, '+' if is_folder else '.', path)
   print '%s  User: %s (%d)' % (indent, r.user_name, r.user_id)
   print '%s  Group: %s (%d)' % (indent, r.group_name, r.group_id)
   print '%s  Permissions: %o' % (indent, r.permissions)  
@@ -37,8 +37,12 @@ def show_entry (r, level=0):
   # otherwise resource is a folder, so recurse into it
   else:
     try:
-      for child in r.children:
-        show_entry (child, level+1)
+      for child in r.get_children ():
+        if path == '/':
+          cpath = path + child.name
+        else:
+          cpath = path + '/' + child.name
+        show_entry (child, cpath, level+1)
     except Exception, e:
       print '%s  <Warning: %s>' % (indent, e)
 
