@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -21,6 +21,7 @@
 //! \author Eduardo Aguiar
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 #include <pymobius.h>
+#include <pygil.h>
 #include "entry.h"
 #include "stream.h"
 #include "io/reader.h"
@@ -89,7 +90,7 @@ tp_getter_size (filesystem_entry_o *self)
 
   try
     {
-      ret = PyLong_FromUnsignedLongLong (self->obj->get_size ());
+      ret = mobius::py::pylong_from_std_uint64_t (self->obj->get_size ());
     }
   catch (const std::exception& e)
     {
@@ -129,7 +130,7 @@ tp_getter_inode (filesystem_entry_o *self)
 
   try
     {
-      ret = PyLong_FromUnsignedLongLong (self->obj->get_inode ());
+      ret = mobius::py::pylong_from_std_uint64_t (self->obj->get_inode ());
     }
   catch (const std::exception& e)
     {
@@ -209,7 +210,7 @@ tp_getter_path (filesystem_entry_o *self)
 
   try
     {
-      ret = PyString_from_std_string (self->obj->get_path ());
+      ret = PyString2_from_std_string (self->obj->get_path ());
     }
   catch (const std::exception& e)
     {
@@ -231,7 +232,7 @@ tp_setter_path (filesystem_entry_o *self, PyObject *value, void *)
       return -1;
     }
 
-  if (!PyString_Check (value))
+  if (!mobius::py::pystring_check (value))
     {
       PyErr_SetString (PyExc_TypeError, "invalid type for 'path' attribute");
       return -1;
@@ -240,7 +241,7 @@ tp_setter_path (filesystem_entry_o *self, PyObject *value, void *)
   // set path
   try
     {
-      self->obj->set_path (PyString_as_std_string (value));
+      self->obj->set_path (mobius::py::pystring_as_std_string (value));
     }
   catch (const std::exception& e)
     {
@@ -261,7 +262,7 @@ tp_getter_name (filesystem_entry_o *self)
 
   try
     {
-      ret = PyString_from_std_string (self->obj->get_name ());
+      ret = PyString2_from_std_string (self->obj->get_name ());
     }
   catch (const std::exception& e)
     {
@@ -281,7 +282,7 @@ tp_getter_short_name (filesystem_entry_o *self)
 
   try
     {
-      ret = PyString_from_std_string (self->obj->get_short_name ());
+      ret = PyString2_from_std_string (self->obj->get_short_name ());
     }
   catch (const std::exception& e)
     {
@@ -301,7 +302,7 @@ tp_getter_creation_time (filesystem_entry_o *self)
 
   try
     {
-      ret = PyDateTime_from_datetime (self->obj->get_creation_time ());
+      ret = mobius::py::pydatetime_from_datetime (self->obj->get_creation_time ());
     }
   catch (const std::exception& e)
     {
@@ -321,7 +322,7 @@ tp_getter_last_modification_time (filesystem_entry_o *self)
 
   try
     {
-      ret = PyDateTime_from_datetime (self->obj->get_last_modification_time ());
+      ret = mobius::py::pydatetime_from_datetime (self->obj->get_last_modification_time ());
     }
   catch (const std::exception& e)
     {
@@ -341,7 +342,7 @@ tp_getter_last_access_time (filesystem_entry_o *self)
 
   try
     {
-      ret = PyDateTime_from_datetime (self->obj->get_last_access_time ());
+      ret = mobius::py::pydatetime_from_datetime (self->obj->get_last_access_time ());
     }
   catch (const std::exception& e)
     {
@@ -361,7 +362,7 @@ tp_getter_last_metadata_time (filesystem_entry_o *self)
 
   try
     {
-      ret = PyDateTime_from_datetime (self->obj->get_last_metadata_time ());
+      ret = mobius::py::pydatetime_from_datetime (self->obj->get_last_metadata_time ());
     }
   catch (const std::exception& e)
     {
@@ -381,7 +382,7 @@ tp_getter_deletion_time (filesystem_entry_o *self)
 
   try
     {
-      ret = PyDateTime_from_datetime (self->obj->get_deletion_time ());
+      ret = mobius::py::pydatetime_from_datetime (self->obj->get_deletion_time ());
     }
   catch (const std::exception& e)
     {
@@ -401,7 +402,7 @@ tp_getter_is_deleted (filesystem_entry_o *self)
 
   try
     {
-      ret = PyBool_FromLong (self->obj->is_deleted ());
+      ret = mobius::py::pybool_from_bool (self->obj->is_deleted ());
     }
   catch (const std::exception& e)
     {
@@ -421,7 +422,7 @@ tp_getter_is_reallocated (filesystem_entry_o *self)
 
   try
     {
-      ret = PyBool_FromLong (self->obj->is_reallocated ());
+      ret = mobius::py::pybool_from_bool (self->obj->is_reallocated ());
     }
   catch (const std::exception& e)
     {
@@ -544,17 +545,15 @@ tp_f_get_children (filesystem_entry_o *self, PyObject *)
   PyObject *ret = nullptr;
 
   // execute C++ code
-  PyThreadState *_save = PyEval_SaveThread ();
-
   try
     {
-      auto children = self->obj->get_children ();
-      PyEval_RestoreThread (_save);
-      ret = PyList_from_cpp_container (children, PyMobius_Filesystem_Entry_from_cpp);
+      auto children = mobius::py::GIL () (
+		        self->obj->get_children ()
+                      );
+      ret = mobius::py::pylist_from_cpp_container (children, PyMobius_Filesystem_Entry_from_cpp);
     }
   catch (const std::exception& e)
     {
-      PyEval_RestoreThread (_save);
       PyErr_SetString (PyExc_Exception, e.what ());
     }
   
@@ -642,7 +641,7 @@ tp_f_get_streams (filesystem_entry_o *self, PyObject *)
 
   try
     {
-      ret = PyList_from_cpp_container (
+      ret = mobius::py::pylist_from_cpp_container (
                self->obj->get_streams (),
                PyMobius_Filesystem_Stream_from_cpp
             );
@@ -685,7 +684,7 @@ tp_f_is_folder (filesystem_entry_o *self, PyObject *)
 
   try
     {
-      ret = PyBool_FromLong (self->obj->is_folder ());
+      ret = mobius::py::pybool_from_bool (self->obj->is_folder ());
     }
   catch (const std::exception& e)
     {
@@ -705,7 +704,7 @@ tp_f_is_file (filesystem_entry_o *self, PyObject *)
 
   try
     {
-      ret = PyBool_FromLong (self->obj->is_file ());
+      ret = mobius::py::pybool_from_bool (self->obj->is_file ());
     }
   catch (const std::exception& e)
     {
