@@ -25,11 +25,15 @@
 #include "application.h"
 #include "case_manager.h"
 #include "case.h"
+#include "chat_message.h"
 #include "cookie.h"
 #include "item.h"
 #include "password.h"
 #include "password_hash.h"
 #include "profile.h"
+#include "text_search.h"
+#include "bookmarked_url.h"
+#include "visited_url.h"
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Module methods
@@ -56,13 +60,17 @@ static constexpr struct
 {
   {"ant", &model_ant_t},
   {"application", &model_application_t},
+  {"bookmarked_url", &model_bookmarked_url_t},
   {"case_manager", &model_case_manager_t},
   {"case", &model_case_t},
+  {"chat_message", &model_chat_message_t},
   {"cookie", &model_cookie_t},
   {"item", &model_item_t},
   {"password", &model_password_t},
   {"password_hash", &model_password_hash_t},
   {"profile", &model_profile_t},
+  {"text_search", &model_text_search_t},
+  {"visited_url", &model_visited_url_t},
 };
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
