@@ -18,37 +18,29 @@
 #include <mobius/core/application.h>
 #include <mobius/io/file.h>
 #include <mobius/io/folder.h>
+#include <mobius/string_functions.h>
 #include <unistd.h>
 #include <iostream>
 #include <iomanip>
 
+#ifdef HAVE_SMB_SUPPORT
+#include <mobius/io/smb/init.h>
+#endif
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief show file info
+//! \brief Show entry metadata
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-void
-show_file (const mobius::io::file& f, int level = 0)
+template <typename F>
+void show_metadata (int level, F f)
 {
-  std::cout << std::endl;
   const std::string indent (level * 2, ' ');
 
-  if (!f || !f.exists ())
-    {
-      std::cout << indent << "? " << f.get_url () << std::endl;
-      return;
-    }
-
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  // show metadata
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  std::cout << indent << ". " << f.get_url () << std::endl;
   std::cout << indent << "  Name: " << f.get_name () << std::endl;
   std::cout << indent << "  Short name: " << f.get_short_name () << std::endl;
   std::cout << indent << "  Inode: " << f.get_inode () << std::endl;
   std::cout << indent << "  Is deleted: " << std::boolalpha << f.is_deleted () << std::endl;
   std::cout << indent << "  Is reallocated: " << std::boolalpha << f.is_reallocated () << std::endl;
   std::cout << indent << "  Is hidden: " << std::boolalpha << f.is_hidden () << std::endl;
-  std::cout << indent << "  Is regular file: " << std::boolalpha << f.is_regular_file () << std::endl;
-  std::cout << indent << "  Size: " << f.get_size () << " bytes" << std::endl;
   std::cout << indent << "  User: " << f.get_user_name () << " (" << f.get_user_id () << ')' << std::endl;
   std::cout << indent << "  Group: " << f.get_group_name () << " (" << f.get_group_id () << ')' << std::endl;
   std::cout << indent << "  Permissions: " << std::oct << f.get_permissions () << std::dec << std::endl;
@@ -58,9 +50,30 @@ show_file (const mobius::io::file& f, int level = 0)
   std::cout << indent << "  Creation time: " << f.get_creation_time () << std::endl;
   std::cout << indent << "  Deletion time: " << f.get_deletion_time () << std::endl;
   std::cout << indent << "  Backup time: " << f.get_backup_time () << std::endl;
+}
 
-  auto parent = f.get_parent ();
-  std::cout << indent << "  Parent folder: " << parent.get_url () << std::endl;
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Show file info
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+show_file (const mobius::io::file& f, const std::string& url, int level = 0)
+{
+  std::cout << std::endl;
+  const std::string indent (level * 2, ' ');
+
+  if (!f || !f.exists ())
+    {
+      std::cout << indent << "? " << url << std::endl;
+      return;
+    }
+
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // show metadata
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  std::cout << indent << ". " << url << std::endl;
+  show_metadata (level, f);
+  std::cout << indent << "  Is regular file: " << std::boolalpha << f.is_regular_file () << std::endl;
+  std::cout << indent << "  Size: " << f.get_size () << " bytes" << std::endl;
 
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   // show some bytes...
@@ -74,7 +87,7 @@ show_file (const mobius::io::file& f, int level = 0)
 
           std::cout << indent << "  First 16 bytes:";
           for (auto b : data)
-            std::cout << ' ' << std::hex << int (b) << std::dec;
+            std::cout << ' ' << mobius::string::to_hex (b, 2);
           std::cout << std::endl;
         }
       catch (const std::runtime_error& e)
@@ -85,30 +98,25 @@ show_file (const mobius::io::file& f, int level = 0)
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief show folder info
+//! \brief Show folder info
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void
-show_folder (const mobius::io::folder& folder, int level = 0)
+show_folder (const mobius::io::folder& folder, const std::string& url = "/", int level = 0)
 {
   std::cout << std::endl;
   const std::string indent (level * 2, ' ');
 
   if (!folder || !folder.exists ())
     {
-      std::cout << indent << "? " << folder.get_url () << std::endl;
+      std::cout << indent << "? " << url << std::endl;
       return;
     }
 
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   // show metadata
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  std::cout << indent << "+ " << folder.get_url () << std::endl;
-  std::cout << indent << "  User: " << folder.get_user_name () << " (" << folder.get_user_id () << ')' << std::endl;
-  std::cout << indent << "  Group: " << folder.get_group_name () << " (" << folder.get_group_id () << ')' << std::endl;
-  std::cout << indent << "  Permissions: " << std::oct << folder.get_permissions () << std::dec << std::endl;
-  std::cout << indent << "  Last access time (atime): " << folder.get_access_time () << std::endl;
-  std::cout << indent << "  Last modification time (mtime): " << folder.get_modification_time () << std::endl;
-  std::cout << indent << "  Last metadata time (ctime): " << folder.get_metadata_time () << std::endl;
+  std::cout << indent << "+ " << url << std::endl;
+  show_metadata (level, folder);
 
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   // handle entries
@@ -118,10 +126,22 @@ show_folder (const mobius::io::folder& folder, int level = 0)
       for (const auto& entry: folder.get_children ())
         {
           if (entry.is_folder ())
-            show_folder (entry.get_folder (), level + 1);
+            {
+              auto cfolder = entry.get_folder ();
+
+              const std::string curl = (url == "/") ? url + cfolder.get_name () : url + '/' + cfolder.get_name ();
+
+              show_folder (cfolder, curl, level + 1);
+            }
 
           else if (entry.is_file ())
-            show_file (entry.get_file (), level + 1);
+            {
+              auto cfile = entry.get_file ();
+
+              const std::string curl = (url == "/") ? url + cfile.get_name () : url + '/' + cfile.get_name ();
+
+              show_file (cfile, curl, level + 1);
+            }
         }
     }
   catch (const std::runtime_error& e)
@@ -140,9 +160,10 @@ usage ()
   std::cerr << "use: dirtree [OPTIONS] <url1> [url2] ..." << std::endl;
   std::cerr << "e.g: dirtree file:///etc" << std::endl;
   std::cerr << std::endl;
-  //std::cerr << "options are:" << std::endl;
-  //std::cerr << "  -o offset\tstarting sector of the filesystem" << std::endl;
-  //std::cerr << std::endl;
+  std::cerr << "options are:" << std::endl;
+  std::cerr << "  -u user\tSMB user name" << std::endl;
+  std::cerr << "  -p password\tSMB user password" << std::endl;
+  std::cerr << std::endl;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -159,42 +180,59 @@ main (int argc, char **argv)
   std::cerr << std::endl;
 
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  // process command line
+  // Process command line
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   int opt;
+  std::string smb_user;
+  std::string smb_password;
 
-  while ((opt = getopt (argc, argv, "")) != EOF)
+  while ((opt = getopt (argc, argv, "u:p:")) != EOF)
     {
-      /*if (opt == 'o')
+      switch (opt)
         {
-          offset = atol (optarg) * 512;
+        case 'h':
+          usage ();
+          exit (EXIT_SUCCESS);
           break;
-        }
-      else
-        {
-          //std::cerr << "Error: Invalid option '-" << char (opt) << "'" << std::endl;
+
+        case 'u':
+          smb_user = optarg;
+          break;
+          
+        case 'p':
+          smb_password = optarg;
+          break;
+        
+        default:
+          std::cerr << "Error: Invalid option '-" << char (opt) << "'" << std::endl;
           usage ();
           exit (EXIT_FAILURE);
-        }*/
+        }
     }
 
   if (optind == argc)
     {
-      std::cerr << "Error: You must pass a valid path" << std::endl;
+      std::cerr << "Error: You must pass a valid url" << std::endl;
       usage ();
       exit (EXIT_FAILURE);
     }
 
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  // show resources
+  // Show entries
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+#ifdef HAVE_SMB_SUPPORT
+  if (!smb_user.empty () || !smb_password.empty ())
+    mobius::io::smb::init (smb_user, smb_password);
+#endif
+
   while (optind < argc)
     {
-      auto folder = mobius::io::new_folder_by_path (argv[optind]);
+      const std::string url = argv[optind];
+      auto folder = mobius::io::new_folder_by_url (url);
 
       try
         {
-          show_folder (folder);
+          show_folder (folder, url);
         }
       catch (const std::exception& e)
         {
